#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include <util\delay.h>

/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRD			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT	PORTD
#define LcdDATA_PIN		PIND

#define LcdCMD_DDR		DDRD			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTD
#define LcdCMD_PIN		PIND

#define LcdD4			PD0				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			PD1				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			PD2				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			PD3				// Pin-ul pentru firul de date D7 de pe LCD

#define LcdRS			PD4				// Pinul pentru selectare operatie (LCD)
#define LcdRW			PD5				// Pinul pentru Read/ Write (LCD)
#define LcdE			PD6				// Pinul de Enable (LCD)

#define LCD_INSTR_4wire 		0x28 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0C 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay 	0x01 	// Clear Display
#define LCD_INSTR_returnHome 	0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position

/************************************************************************ 
 * DEFINE-uri pentru primire, transmitere semnal
 ************************************************************************/
 
#define DSTATUS		DDRC
#define DSEND		DDRA
#define DRECEIVE	DDRB
#define ISTATUS		PINC
#define ISEND		PINA
#define IRECEIVE	PINB
#define OSTATUS		PORTC
#define OSEND		PORTA
#define ORECEIVE	PORTB
#define CSTRAIGHT	128
#define CCROSS		64
#define CROLLOVER	32
#define CBAD		16

/************************************************************************ 
 * API LCD.. implementare este in partea de jos a fisierului
 ************************************************************************/
void LCD_init();											// Initializare modul LCD.. trebuie apelata inainte de a se face orice operatie cu LCD-ul
void LCD_writeInstruction(unsigned char _instruction);	// Trimite o instructiune catre lcd (vezi datasheet)
void LCD_writeData(unsigned char _data);					// Trimite date catre LCD pentru afisare
void LCD_writeData(unsigned char _data);					// Trimite date catre LCD pentru afisare
void LCD_write(unsigned char _byte);						// trimite un bute catre LCD la modul general (nu conteaza daca e instructiune sau date)
void LCD_waitNotBusy();									// Asteptam pana cand lcd-ul devine disponibil pt o noua comanda
void LCD_print(char* _msg);								// Afiseaza imformatia pe LCD (doar 1 linie, primele 16 caractere)
void LCD_print2(char* _msg1, char* _msg2);				// Afisare pe 2 lini pe LCD
void LCD_printDecimal2u(unsigned int _n);					// Afisare numar in baza 10 pe LCD
void LCD_printHexa(unsigned int _n);						// Afisare numar in baza 16 pe LCD
//void LCD_waitInstructions(unsigned char _instructions);	// Asteapta un numar de cicli de tact.. loop

// tipuri de cablu
int STRAIGHT[8]=		{1, 2, 3, 4, 5, 6, 7, 8 };
int CROSS[8]=			{3, 6, 1, 4, 5, 2, 7, 8 };
int CROSSGB[8]=			{3, 6, 1, 7, 8, 2, 4, 5 };
int ROLLOVER[8]=		{8, 7, 6, 5, 4, 3, 2, 1 };
// vectorul in care se face citirea
int READ[8]=		{0, 0, 0, 0, 0, 0, 0, 0 };
// al cate-lea pin este aprins
int which(int x){
    int i=0;
	while(x!=0){
		x=x>>1;
		i++;
	}
	return i;
}
// sunt egali cei doi vectori ?
int eq(int *a, int *b){
	int i;
	for(i=0; i<7; i++)
		if(a[i]!=b[i])
			return 0;
	return 1;
}
int main(){
	int i, r;
	unsigned long long s;
	// setam registrii de directie
	DSTATUS = 0xFF;
	DSEND = 0xFF;
	DRECEIVE = 0x00;
	
	LCD_init();
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	
	while(1)
	{
		// testez cei 8 pini trimitand pe fiecare
		for(i=0; i<8; i++){
			OSEND=1<<i;
			_delay_ms(100);
			// si vad pe ce prin am primit
			r=IRECEIVE;
			OSTATUS=r;
			// si memorez in vector
			READ[i]=which(r);
			_delay_ms(500);
		}
		// aprind becul corespunzator tipului de cablu
		if(eq(READ, STRAIGHT))
			LCD_print("Cablu STRAIGHT");
		else if(eq(READ, CROSS) || eq(READ, CROSSGB))
			LCD_print("Cablu CROSSOVER");
		else if(eq(READ, ROLLOVER))
			LCD_print("Cablu ROLLOVER");
		else 
			LCD_print("Problema cablu");
		// wait
		OSTATUS=0x00;
		OSEND=0x00;
		_delay_ms(1000);		
	};
	return 0;
}

/************************************************************************ 
 * IMPLEMENTARE API LCD.. implementare este in partea de jos a fisierului
 ************************************************************************/

void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Sepcificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	//LCD_waitInstructions(6);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display
}

void LCD_writeInstruction(unsigned char _instruction)
{
	LCD_waitNotBusy();							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	LcdCMD_PORT &= ~(1<<LcdRS);				// setam pinul RS pe low.. low=instructiuni, high=date
	LcdCMD_PORT &= ~(1<<LcdRW);				// setam pinu RW pe low (suntem in modul de comenzi acum)
	LCD_write(_instruction);					// apelam procedura ce trimite bute-ul pe firele de date
}


void LCD_writeData(unsigned char _data)
{
	LCD_waitNotBusy();							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile
	LcdCMD_PORT |=  (1<<LcdRS);				// setam pinul RS pe high
	LcdCMD_PORT &= ~(1<<LcdRW);				// setam pinu RW pe low (suntem in modul de date acum)
	LCD_write(_data);							// apelam procedura ce trimite bute-ul pe firele de date
}

void LCD_write(unsigned char _byte)
{
	unsigned char _byte2;
	
	_byte2 = _byte>>4;

	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte2, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
		
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte2, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte2, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte2, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

	//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
	
	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
	
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
}


void LCD_waitNotBusy()
{
	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		//LCD_waitInstructions(6);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}
}




void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0;
	
	// Extragem sutele
	while(_n>=100)
		_n-=100;

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	LCD_writeData(tmp+'0');
	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}